package com.zskk.dicom.monitor.oss;

import cn.ucloud.ufile.UfileClient;
import cn.ucloud.ufile.api.object.ObjectConfig;
import cn.ucloud.ufile.api.object.policy.PolicyParam;
import cn.ucloud.ufile.api.object.policy.PutPolicy;
import cn.ucloud.ufile.api.object.policy.PutPolicyForCallback;
import cn.ucloud.ufile.auth.ObjectAuthorization;
import cn.ucloud.ufile.auth.UfileObjectLocalAuthorization;
import cn.ucloud.ufile.bean.PutObjectResultBean;
import cn.ucloud.ufile.exception.UfileClientException;
import cn.ucloud.ufile.exception.UfileServerException;
import cn.ucloud.ufile.http.OnProgressListener;
import cn.ucloud.ufile.util.JLog;
import cn.ucloud.ufile.util.MimeTypeUtil;
import cn.ucloud.ufile.util.StorageType;
import com.aliyun.oss.common.utils.IOUtils;
import com.zskk.dicom.monitor.config.Configs;
import org.apache.commons.codec.digest.DigestUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

public class LcUfileUtils {

    private static final String PUBLIC_KEY = "TOKEN_29e194bf-beff-4155-a205-daf0feddcf65";
    private static final String PRIVATE_KEY = "61e2695b-42a0-4190-a460-a2031325b9d2";

    private static final ObjectAuthorization OBJECT_AUTHORIZER = new UfileObjectLocalAuthorization(PUBLIC_KEY, PRIVATE_KEY);

    private static final String REGION = "cn-inspur2";
    private static final String PORXY_SUFFIX = "ufileos.com";
    private static final String CUSTOM_HOST = "zskkdcm.infile.inspurcloud.cn";

    private static final String TO_BUCKET = "zskkdcm";
    private static final ObjectConfig config = new ObjectConfig("api.spark.ucloud.cn");
//    private static final ObjectConfig config = new ObjectConfig("www.cn-bj.ufileos.com");
//    private static final ObjectConfig config = new ObjectConfig(REGION, PORXY_SUFFIX);
//    static  {
//        config.setCustomHost(CUSTOM_HOST);
//    }


    public static boolean upload(File file) {
        FileInputStream fis = null;
        String md5 = null;
        String fileName = null;
        long length = -1;
        try {
            fis = new FileInputStream(file);
            md5 = DigestUtils.md5Hex(IOUtils.readStreamAsByteArray(fis));
            fileName = Configs.hospitalId + "/" + md5;
//			fileName = Configs.hospitalId + File.separator + md5;
            length = file.length();
            fis.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return putFile(file, MimeTypeUtil.getMimeType("application/dicom"), md5, length);
    }


    public static boolean putFile(File file, String mimeType, String nameAs, long length) {
        try {
            /**
             * 上传回调策略
             * 必须填写回调接口url(目前仅支持http，不支持https)，可选填回调参数，回调参数请自行决定是否需要urlencode
             * 若配置上传回调，则上传接口的回调将会透传回调接口的response，包括httpCode
             */
            PutPolicy putPolicy = new PutPolicyForCallback.Builder(Configs.lcossCallbackUrl)
                    .addCallbackBody(new PolicyParam("key", "value"))
                    .build();
            PutObjectResultBean response = UfileClient.object(OBJECT_AUTHORIZER, config)
                    .putObject(file, mimeType)
                    .nameAs(nameAs)
                    .toBucket(TO_BUCKET)
                    /**
                     * 配置文件存储类型，分别是标准、低频、冷存，对应有效值：STANDARD | IA | ARCHIVE
                     */
                    .withStorageType(StorageType.STANDARD)
                    /**
                     * 为云端对象配置自定义数据，每次调用将会替换之前数据。
                     * 所有的自定义数据总大小不能超过 8KB。
                     */
//                    .withMetaDatas()
                    /**
                     * 为云端对象添加自定义数据，可直接调用，无须先调用withMetaDatas
                     * key不能为空或者""
                     *
                     */
//                    .addMetaData(new Parameter<>("key","value"))
                    /**
                     * 配置上传回调策略
                     */
//                .withPutPolicy(putPolicy)
                    /**
                     * 是否上传校验MD5
                     */
//                .withVerifyMd5(false)
                    /**
                     * 指定progress callback的间隔
                     */
//                .withProgressConfig(ProgressConfig.callbackWithPercent(10))
                    /**
                     * 配置读写流Buffer的大小, Default = 256 KB, MIN = 4 KB, MAX = 4 MB
                     */
//                    .setBufferSize(4 << 20)
                    /**
                     * 配置进度监听
                     */
                    .setOnProgressListener(new OnProgressListener() {
                        @Override
                        public void onProgress(long bytesWritten, long contentLength) {
                            JLog.D("TAG", String.format("[progress] = %d%% - [%d/%d]", (int) (bytesWritten * 1.f / contentLength * 100), bytesWritten, contentLength));
                        }
                    })
                    .execute();
                    Configs.sysLog.info("response" + response);
//            JLog.D("TAG", String.format("[res] = %s", (response == null ? "null" : response.toString())));
        } catch (UfileClientException e) {
            e.printStackTrace();
        } catch (UfileServerException e) {
            e.printStackTrace();
        }
        return true;
    }
}
