package com.zskk.dicom.oss;

import com.alibaba.fastjson.JSON;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.common.comm.ResponseMessage;
import com.aliyun.oss.common.utils.IOUtils;
import com.aliyun.oss.event.ProgressEvent;
import com.aliyun.oss.event.ProgressEventType;
import com.aliyun.oss.event.ProgressListener;
import com.aliyun.oss.model.AppendObjectRequest;
import com.aliyun.oss.model.AppendObjectResult;
import com.zskk.dicom.monitor.config.Configs;
import com.zskk.dicom.monitor.report.ErrReporter;
import com.zskk.dicom.monitor.utils.ExceptionUtil;
import com.zskk.dicom.response.BaseResponse;
import com.zskk.dicom.response.bean.FileUploadResponseBean;
import org.apache.commons.codec.digest.DigestUtils;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;

public class OSSAddUploadHleper implements BaseOSSHleper {

	private OSSClient instance;
	public OSSAddUploadHleper(String ossEndpoint , String accessKeyId , String accessKeySecret ) {
		instance = new OSSClient(ossEndpoint,accessKeyId,accessKeySecret);
	}
	@Override
	public boolean exist(String md5) {
		return instance.doesObjectExist(Configs.ossBucketName, md5);
	}
	
	@Override
	public boolean upload(File file) {
		BaseResponse exception = null;
		FileInputStream fis = null;
		String md5 = null;
		Exception error = null;
		try {
			fis = new FileInputStream(file);
			md5 = DigestUtils.md5Hex(IOUtils.readStreamAsByteArray(fis));
			fis.close();
		} catch (IOException e) {
			ErrReporter.report(ExceptionUtil.getExceptionTxt(e));
//			e.printStackTrace();
			return false;
		}
		String fileName = Configs.hospitalId + "/" + md5;
		try {
			if(instance.doesObjectExist(Configs.ossBucketName, fileName)) {
				Configs.sysLog.info(fileName + "文件存在, 开始删除");
				instance.deleteObject(Configs.ossBucketName, fileName);
				Configs.sysLog.info(fileName + "文件删除成功开始上传");
			}

			InputStream in = new FileInputStream(file);
			byte[] buffer = new byte[1024];
			int bytes = 0;
			int count = 0;
			bytes = in.read(buffer);
			InputStream out = new ByteArrayInputStream(buffer, 0, bytes);
			AppendObjectRequest appendObjectRequest = new AppendObjectRequest(Configs.ossBucketName, fileName, out);
			appendObjectRequest.setPosition(0L);
			AppendObjectResult appendObjectResult = instance.appendObject(appendObjectRequest);
			while ((bytes = in.read(buffer)) != -1) {
				appendObjectRequest.setPosition(appendObjectResult.getNextPosition());
				appendObjectRequest.setInputStream(new ByteArrayInputStream(buffer, 0, bytes));
				appendObjectResult = instance.appendObject(appendObjectRequest);
				Configs.sysLog.info("追加上传" + fileName + "-" + count++ + "");
			}
			Configs.sysLog.info("上传完成 responseMessage");
			instance.shutdown();
			instance = null;
		} catch (FileNotFoundException e) {
			Configs.sysLog.info("FileNotFoundException" + ExceptionUtil.getExceptionTxt(e));
			return  false;
		} catch (IOException e) {
			Configs.sysLog.info("IOException" + ExceptionUtil.getExceptionTxt(e));
			return  false;
		}
		Configs.sysLog.info("上传完成, 开始构建oss回调");

		URL url = null;
		try {
			url = new URL(Configs.ossCallbackUrl);
			HttpURLConnection connection = (HttpURLConnection) url.openConnection();
			connection.setRequestMethod("POST");
			connection.setDoOutput(true);
			connection.setDoInput(true);
			connection.setUseCaches(false);
			connection.setRequestProperty("x-oss-bucket", Configs.ossBucketName);
			connection.connect();
			String body = "md5="+fileName+"&url=dicomweb://zskk-dcm.oss-cn-beijing.aliyuncs.com/"+fileName+ "&hospital_id=" + Configs.hospitalId + "&size=" + file.length() +"&region="+ Configs.ossRegion + "&bucket=" + Configs.ossBucketName;
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(connection.getOutputStream(), "UTF-8"));
			writer.write(body);
			writer.close();
			int responseCode = connection.getResponseCode();
			Configs.sysLog.warn("回调请求成功 开始解析返回结果 responseCode" + responseCode + Configs.ossCallbackUrl);
			if(responseCode == HttpURLConnection.HTTP_OK){
				StringBuilder strBuf = new StringBuilder();
				BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
				try {
					String line = null;
					while ((line = reader.readLine()) != null) {
						strBuf.append(line).append("\n");
					}
				} finally {
					reader.close();
				}
				String res = strBuf.toString();
				Configs.sysLog.warn("aliyunoss upload res" + res);
				FileUploadResponseBean fileUploadResponseBean = JSON.parseObject(res, FileUploadResponseBean.class);
				// 数据读取完成后，获取的流必须关闭，否则会造成连接泄漏，导致请求无连接可用，程序无法正常工作。
				// 关闭OSSClient。
				if(fileUploadResponseBean.code == 0) {
					return true;
				}

				if(fileUploadResponseBean.code == 1002) {
					ErrReporter.report("OSS回调失败:md5" + md5 + "返回结果" + fileUploadResponseBean.toString() + "1002 文件已存在认为成功");
					return true;
				}
				ErrReporter.report("OSS回调失败:md5" + md5 + "返回结果" + fileUploadResponseBean.toString());
				return false;
			}
		} catch (MalformedURLException e) {
			Configs.sysLog.info("MalformedURLException" + ExceptionUtil.getExceptionTxt(e));
			return  false;
		} catch (UnsupportedEncodingException e) {
			Configs.sysLog.info("UnsupportedEncodingException" + ExceptionUtil.getExceptionTxt(e));
			return  false;
		} catch (ProtocolException e) {
			Configs.sysLog.info("ProtocolException" + ExceptionUtil.getExceptionTxt(e));
			return  false;
		} catch (IOException e) {
			Configs.sysLog.info("IOException" + ExceptionUtil.getExceptionTxt(e));
			return  false;
		}
		return false;
	}
}
